
from flask import Flask, redirect, request, render_template, url_for, session
import requests
import os
import time
import db

app = Flask(__name__)
app.secret_key = 'your-very-secure-secret-key'  # CHANGE in production!

CLIENT_ID = os.environ.get('CLIENT_ID', '2flghubBfH3vMLHlkd5dcsZSD8J6eiUxGuleRVRriAD5UNR')
CLIENT_SECRET = os.environ.get('CLIENT_SECRET', 'Zw1FhhvdOAxZMd1qPKjAbODdFzRbjooPfqTtzodbTJMZ0dhg')
REDIRECT_URI = os.environ.get('REDIRECT_URI', 'https://yourdomain.com/callback')
SCOPE = 'read,write'

AUTH_URL = 'https://noones.com/oauth/authorize'
TOKEN_URL = 'https://noones.com/oauth/token'
API_BASE = 'https://noones.com/api/v1'

db.init_db()

@app.route('/')
def index():
    params = {
        'client_id': CLIENT_ID,
        'response_type': 'code',
        'redirect_uri': REDIRECT_URI,
        'scope': SCOPE,
        'state': 'random_state_string'  # Change to a secure random string
    }
    auth_url = f"{AUTH_URL}?{'&'.join(f'{k}={v}' for k,v in params.items())}"
    return render_template('index.html', auth_url=auth_url)

@app.route('/callback')
def callback():
    error = request.args.get('error')
    if error:
        return f"Error during authorization: {error}"

    code = request.args.get('code')
    if not code:
        return "No authorization code provided."

    data = {
        'grant_type': 'authorization_code',
        'code': code,
        'redirect_uri': REDIRECT_URI,
        'client_id': CLIENT_ID,
        'client_secret': CLIENT_SECRET
    }
    resp = requests.post(TOKEN_URL, data=data)
    if resp.status_code != 200:
        return f"Failed to get token: {resp.status_code} {resp.text}"

    tokens = resp.json()
    access_token = tokens['access_token']
    refresh_token = tokens.get('refresh_token')
    expires_in = tokens.get('expires_in', 3600)

    session_id = session.get('session_id')
    if not session_id:
        session_id = str(time.time()) + request.remote_addr
        session['session_id'] = session_id

    expires_at = time.time() + expires_in - 60
    db.save_tokens(session_id, access_token, refresh_token, expires_at)

    return redirect(url_for('profile'))

def get_valid_access_token(session_id):
    token_data = db.load_tokens(session_id)
    if not token_data:
        return None

    if time.time() > token_data['expires_at']:
        refresh_data = {
            'grant_type': 'refresh_token',
            'refresh_token': token_data['refresh_token'],
            'client_id': CLIENT_ID,
            'client_secret': CLIENT_SECRET
        }
        resp = requests.post(TOKEN_URL, data=refresh_data)
        if resp.status_code != 200:
            return None

        new_tokens = resp.json()
        token_data['access_token'] = new_tokens['access_token']
        token_data['refresh_token'] = new_tokens.get('refresh_token', token_data['refresh_token'])
        expires_in = new_tokens.get('expires_in', 3600)
        token_data['expires_at'] = time.time() + expires_in - 60
        db.save_tokens(session_id, token_data['access_token'], token_data['refresh_token'], token_data['expires_at'])

    return token_data['access_token']

@app.route('/profile')
def profile():
    session_id = session.get('session_id')
    if not session_id:
        return redirect(url_for('index'))

    access_token = get_valid_access_token(session_id)
    if not access_token:
        return "Unable to refresh or get access token. Please re-authenticate."

    headers = {'Authorization': f'Bearer {access_token}'}
    resp = requests.get(f'{API_BASE}/user', headers=headers)

    if resp.status_code != 200:
        return f"Failed to get user info: {resp.status_code} {resp.text}"

    user_info = resp.json()

    return render_template('profile.html', user=user_info)

@app.route('/trade', methods=['POST'])
def trade():
    session_id = session.get('session_id')
    if not session_id:
        return "Unauthorized", 401

    access_token = get_valid_access_token(session_id)
    if not access_token:
        return "Authentication required", 401

    trade_data = {
        "symbol": "AAPL",
        "side": "buy",
        "quantity": 1,
        "order_type": "market"
    }

    headers = {'Authorization': f'Bearer {access_token}', 'Content-Type': 'application/json'}
    resp = requests.post(f'{API_BASE}/trades', headers=headers, json=trade_data)

    if resp.status_code == 201:
        return f"Trade placed successfully: {resp.json()}"
    else:
        return f"Trade failed: {resp.status_code} {resp.text}", resp.status_code

if __name__ == '__main__':
    app.run(debug=True)
